﻿using System;
using System.Linq;
using System.Net.Mime;
using System.Web;
using System.Web.Mvc;
using DynamicsCRM.Models;
using DynamicsCRM.Services;
using Microsoft.Xrm.Client;
using Orchard;
using Orchard.Themes;
using Xrm;

namespace DynamicsCRM.Controllers {
    [Themed]
    public class CaseController : Controller {
        private const string HomeUrl = "~/DynamicsCRM/Home";
        private const string RelUrl = "~/DynamicsCRM/Case";
        private static Guid _crmUserId = Guid.Empty;
        private readonly IXrmRepository _repository;

        public CaseController(IOrchardServices orchardServices) {
            _repository = new XrmRepository(orchardServices);
        }

        public CaseController(IXrmRepository repository) {
            _repository = repository;
        }

        [Authorize]
        public ActionResult Index() {
            //Retrieve the current contact
            _crmUserId = _repository.RetrieveMyIdByIdentifier();

            //Get a list of cases
            if (_crmUserId != Guid.Empty) {
                var incidents = _repository.RetrieveIncidents(_crmUserId, null);
                if (incidents != null && incidents.Any()) {
                    ViewData["incidents"] = incidents;
                }
                return View();
            }
            TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
            return Redirect(HomeUrl);
        }

        [Authorize]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Index(CaseModel model) {
            return View(model);
        }

        [Authorize]
        public ActionResult Create() {
            //Retrieve the current contact
            _crmUserId = _repository.RetrieveMyIdByIdentifier();
            if (_crmUserId != Guid.Empty) {
                return View();
            }

            TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
            return Redirect(HomeUrl);
        }

        [Authorize]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(CaseModel model, string button) {
            if (button == "cancel") {
                return Redirect(RelUrl);
            }

            if (ModelState.IsValid) {
                if (_crmUserId != Guid.Empty) {
                    //Create a new incident
                    var incident = new Incident {
                        Title = model.Title,
                        CaseTypeCode = Convert.ToInt32(model.CaseType),
                        CustomerId = new CrmEntityReference("contact", _crmUserId),
                        Description = model.Description
                    };
                    _repository.CreateCrmRecord(incident);

                    TempData["UserMessage"] = "A new case has been created.";
                    return Redirect(RelUrl);
                }

                TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
                return Redirect(HomeUrl);
            }

            ModelState.AddModelError("", "CRM case create error.");
            return View(model);
        }

        [Authorize]
        public ActionResult Update(string ticketNumber) {
            //Retrieve the current contact
            _crmUserId = _repository.RetrieveMyIdByIdentifier();

            //Get the cases
            if (_crmUserId != Guid.Empty) {
                var model = new CaseUpdateModel();
                if (ticketNumber != null) {
                    var incidents = _repository.RetrieveIncidents(_crmUserId, ticketNumber);
                    if (incidents != null && incidents.Count() == 1) {
                        var incident = incidents.SingleOrDefault();

                        model.IncidentId = incident.Id;
                        model.TicketNumber = incident.TicketNumber;
                        model.Title = incident.Title;
                        model.CaseStatus = incident.CaseStatus;
                        model.CaseType = incident.CaseType;
                        model.Description = incident.Description;

                        var notes = _repository.RetrieveNotes(new Guid(incident.Id), Guid.Empty);
                        if (notes != null && notes.Any()) {
                            ViewData["notes"] = notes;
                        }
                    }
                }

                return View(model);
            }

            TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
            return Redirect(HomeUrl);
        }

        [Authorize]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Update(CaseUpdateModel model, string button, HttpPostedFileBase file) {
            if (button == "cancel") {
                return Redirect(RelUrl);
            }

            if (ModelState.IsValid) {
                //Retrieve the current contact
                _crmUserId = _repository.RetrieveMyIdByIdentifier();

                if (_crmUserId != Guid.Empty) {
                    //Create note
                    _repository.CreateNote(model.Note, new Guid(model.IncidentId), "incident", file);
                    TempData["UserMessage"] = "The case has been updated.";

                    return Redirect(RelUrl);
                }

                TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
                return Redirect(HomeUrl);
            }

            ModelState.AddModelError("", "CRM case update error.");
            return View(model);
        }

        [Authorize]
        public ActionResult NoteAttachment(string incidentId, string noteId) {
            if (incidentId != null && noteId != null) {
                var notes = _repository.RetrieveNotes(new Guid(incidentId), new Guid(noteId));

                if (notes != null && notes.Count() == 1) {
                    var note = notes.SingleOrDefault();

                    if (note != null) {
                        var noteBody = Convert.FromBase64String(note.DocumentBody);
                        return File(noteBody, MediaTypeNames.Application.Octet, note.FileName);
                    }
                }
            }

            return View();
        }
    }
}