﻿using System;
using System.Linq;
using System.Web.Mvc;
using DynamicsCRM.Models;
using DynamicsCRM.Services;
using Microsoft.Xrm.Client;
using Orchard;
using Orchard.Themes;
using Xrm;

namespace DynamicsCRM.Controllers {
    [Themed]
    public class CustomerController : Controller {
        private const string HomeUrl = "~/DynamicsCRM/Home";
        private const string RelUrl = "~/DynamicsCRM/Customer";
        private static Guid _crmUserId = Guid.Empty;
        private readonly IXrmRepository _repository;

        public CustomerController(IOrchardServices orchardServices) {
            _repository = new XrmRepository(orchardServices);
        }

        public CustomerController(IXrmRepository repository) {
            _repository = repository;
        }

        [Authorize]
        public ActionResult Index() {
            //Retrieve the current contact
            _crmUserId = _repository.RetrieveMyIdByIdentifier();

            //Get a list of sub-contacts
            if (_crmUserId != Guid.Empty) {
                var customers = _repository.RetrieveCustomers(_crmUserId, Guid.Empty, false);
                if (customers != null && customers.Any()) {
                    ViewData["customers"] = _repository.ListCustomers(customers);
                }

                return View();
            }

            TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
            return Redirect(HomeUrl);
        }

        [Authorize]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Index(CustomerModel model) {
            return View();
        }

        [Authorize]
        public ActionResult Create() {
            //Retrieve the current contact
            _crmUserId = _repository.RetrieveMyIdByIdentifier();
            if (_crmUserId != Guid.Empty) {
                return View();
            }

            TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
            return Redirect(HomeUrl);
        }

        [Authorize]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(CustomerModel model, string button) {
            if (button == "cancel") {
                return Redirect(RelUrl);
            }

            if (ModelState.IsValid) {
                _crmUserId = _repository.RetrieveMyIdByIdentifier();

                if (_crmUserId != Guid.Empty) {
                    //Create a new contact
                    var customer = new Contact {
                        ParentCustomerId = new CrmEntityReference("contact", _crmUserId),
                        EMailAddress1 = model.EmailAddress,
                        FirstName = model.FirstName,
                        LastName = model.LastName,
                        Address1_Name = model.AddressName,
                        Address1_Line1 = model.AddressLine1,
                        Address1_Line2 = model.AddressLine2,
                        Address1_Line3 = model.AddressLine3,
                        Address1_City = model.City,
                        Address1_County = model.County,
                        Address1_Country = model.Country,
                        Address1_PostalCode = model.PostCode,
                        Address1_Telephone1 = model.Telephone
                    };

                    _repository.CreateCrmRecord(customer);
                    TempData["UserMessage"] = "The customer information has been created.";

                    return View(model);
                }

                TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
                return Redirect(HomeUrl);
            }

            ModelState.AddModelError("", "CRM customer create error.");
            return View(model);
        }

        [Authorize]
        public ActionResult Update(string customerId) {
            //Retrieve the current contact
            _crmUserId = _repository.RetrieveMyIdByIdentifier();

            //Get the customer
            if (_crmUserId != Guid.Empty) {
                var model = new CustomerModel();
                if (customerId != null) {
                    var customers = _repository.RetrieveCustomers(_crmUserId, new Guid(customerId), true);
                    if (customers != null && customers.Count() == 1) {
                        var customer = customers.SingleOrDefault();
                        if (customer != null) {
                            model.CustomerId = customerId;
                            model.EmailAddress = customer.EMailAddress1;
                            model.FirstName = customer.FirstName;
                            model.LastName = customer.LastName;
                            model.AddressName = customer.Address1_Name;
                            model.AddressLine1 = customer.Address1_Line1;
                            model.AddressLine2 = customer.Address1_Line2;
                            model.AddressLine3 = customer.Address1_Line3;
                            model.City = customer.Address1_City;
                            model.County = customer.Address1_County;
                            model.Country = customer.Address1_Country;
                            model.PostCode = customer.Address1_PostalCode;
                            model.Telephone = customer.Address1_Telephone1;
                        }
                    }
                }

                return View(model);
            }

            TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
            return Redirect(HomeUrl);
        }

        [Authorize]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Update(CustomerModel model, string button) {
            if (button == "cancel") {
                return Redirect(RelUrl);
            }

            if (ModelState.IsValid) {
                //Retrieve the customer
                _crmUserId = _repository.RetrieveMyIdByIdentifier();

                //Get the customer
                if (_crmUserId != Guid.Empty) {
                    //Create a new customer
                    var customer = new Contact {
                        ContactId = new Guid(model.CustomerId),
                        EMailAddress1 = model.EmailAddress,
                        FirstName = model.FirstName,
                        LastName = model.LastName,
                        Address1_Name = model.AddressName,
                        Address1_Line1 = model.AddressLine1,
                        Address1_Line2 = model.AddressLine2,
                        Address1_Line3 = model.AddressLine3,
                        Address1_City = model.City,
                        Address1_County = model.County,
                        Address1_Country = model.Country,
                        Address1_PostalCode = model.PostCode,
                        Address1_Telephone1 = model.Telephone
                    };

                    _repository.UpdateCrmRecord(customer);
                    TempData["UserMessage"] = "The customer information has been updated.";

                    return View(model);
                }

                TempData["UserMessage"] = "You don't have a CRM account yet - please create your CRM account.";
                return Redirect(HomeUrl);
            }

            ModelState.AddModelError("", "CRM customer update error.");
            return View(model);
        }
    }
}